import requests
from bs4 import BeautifulSoup

from coupon.scrappers.basescrapper import (all_stores, BaseScrapingClass)
from coupon.scrappers.constants import URL_1001


class CupomDesconto1001(BaseScrapingClass):
    base_url = 'https://www.1001cupomdedescontos.com.br{}'

    def __init__(self, url):
        self.header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        r = requests.get(url, headers=self.header, timeout=60000)
        self.soup = BeautifulSoup(r.content, "lxml")
        store_coupons = self.soup.find('div', class_='tab-content wlt_search_results list_style')
        self.valid_coupons = store_coupons.findAll('div', class_='thumbnail clearfix')
        self.store_name = self.soup.find('strong').contents[0]

    def _is_valid_coupon(self, coupon):
        # filter invalid coupons
        atempt1 = coupon.upper() not in ['VERIFIQUE NO SITE', 'CONFIRA NO SITE', '', 'VERIFIQUE']
        atempt2 = '1001' in coupon.upper()
        return atempt1 or atempt2

    def _is_valid_store(self, store_name):
        # to check if all coupons from a certain page are
        # from same store
        return store_name.replace(' ', '') in str(self.store_name).replace(' ', '')

    @classmethod
    def get_url_for_store(cls, store_name):
        store_dict = all_stores.get(store_name)
        if not store_dict:
            return
        resource_url = store_dict.get(URL_1001)
        if not resource_url:
            return
        return cls.base_url.format(resource_url)

    # @classmethod
    # def get_url_for_category(cls, category_name):
    #     category_dict = feature_categories.get(category_name)
    #     if not category_dict:
    #         return
    #     return cls.base_url.format(category_dict.get(URL_1001))

    @property
    def store_coupons(self):
        d = {}
        for coupon_tag in self.valid_coupons:
            expire_date = coupon_tag.find('span', class_='testedtoday').contents[0]
            store_name = coupon_tag.find('span', class_='wlt_shortcode_store').contents[0].strip().replace('Cupom ','')
            description = coupon_tag.find('div', class_='col-md-9 wrap').contents[3].contents[0]
            rules_at1 = coupon_tag.find('p', class_='item-title')
            rules_at2 = coupon_tag.find('div', class_='core_post_content')
            rules = ''
            if rules_at1:
                rules = rules_at1.contents[0]
            elif rules_at2:
                rules = rules_at2.contents[0]
                if not isinstance(rules, str):
                    rules = rules_at2.contents[0].contents[0]
            coupon_at1 = coupon_tag.find('a', class_='btn btncode')
            coupon = ''
            if coupon_at1:
                coupon = coupon_at1.contents[1]
            if self._is_valid_coupon(coupon):
                store_name = self._fix_store_name(store_name)
                if not self._is_valid_store(store_name):
                    continue
                d[coupon] = dict(description=description,
                                 rules=rules,
                                 store_name=store_name,
                                 store_id='',
                                 store_url='',
                                 expire_date=expire_date)
        return d

    @property
    def category_coupons(self):
        d = {}
        for coupon_tag in self.valid_coupons:
            expire_date = coupon_tag.find('span', class_='testedtoday').contents[0]
            store_name = coupon_tag.find('span', class_='wlt_shortcode_store').contents[1].contents[0].replace('Cupom ',
                                                                                                               '')
            description = coupon_tag.find('div', class_='col-md-9 wrap').contents[3].contents[0]
            rules_at1 = coupon_tag.find('p', class_='item-title')
            rules_at2 = coupon_tag.find('div', class_='core_post_content')
            rules = ''
            if rules_at1:
                rules = rules_at1.contents[0]
            elif rules_at2:
                rules = rules_at2.contents[0]
                if not isinstance(rules, str):
                    rules = rules_at2.contents[0].contents[0]
            coupon_at1 = coupon_tag.find('a', class_='btn btncode')
            coupon = ''
            if coupon_at1:
                coupon = coupon_at1.contents[1]
            if self._is_valid_coupon(coupon):
                store_name = self._fix_store_name(store_name)
                d[coupon] = dict(description=description,
                                 rules=rules,
                                 store_name=store_name,
                                 store_id='',
                                 store_url='',
                                 expire_date=expire_date)
        return d

    @classmethod
    def get_store_list(cls):
        d = {}
        letters = "0-9", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", \
                  "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"
        header = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 '
                                '(KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        for letter in letters:
            current_url = cls.base_url.format('/cupom-por-loja?ap={}'.format(letter))
            r = requests.get(current_url, headers=header, timeout=60000)
            soup = BeautifulSoup(r.content, "lxml")
            stores = soup.find('div', class_='taxonomygrid').find_all('li')
            for store in stores:
                store_data = store.find('a')
                store_url = store_data.attrs.get('href')
                store_name = store_data.attrs.get('title').strip()
                d[store_name] = store_url
        return d
