import requests
from bs4 import BeautifulSoup

from coupon.scrappers.basescrapper import all_stores, BaseScrapingClass
from coupon.scrappers.constants import URL_CUPOMZEIROS


class Cupomzeiros(BaseScrapingClass):
    base_url = 'https://www.cupomzeiros.com.br{}'

    def __init__(self, url):
        self.header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        r = requests.get(url, headers=self.header, timeout=60000)
        self.soup = BeautifulSoup(r.content, "lxml")
        self._valid_coupons = self.soup.findAll('div',
                                                class_='coupon-item has-thumb store-listing-item c-type-code coupon-listing-item shadow-box coupon-live')

    def _get_store_name(self):
        store_name = self.soup.find('title').contents[0].strip(' - Cupomzeiros')
        store_name = self._fix_store_name(store_name)
        return store_name

    @classmethod
    def get_url_for_store(cls, store_name):
        store_dict = all_stores.get(store_name)
        if not store_dict:
            return
        resource_url = store_dict.get(URL_CUPOMZEIROS)
        if not resource_url:
            return
        return cls.base_url.format(resource_url)

    @classmethod
    def get_url_for_category(cls, category_name):
        return None

    @property
    def store_coupons(self):
        d = {}
        for box in self._valid_coupons:
            latest_coupon = box.find('div', class_='latest-coupon')
            description = latest_coupon.find('a', class_='coupon-link').attrs.get('title')
            coupon = latest_coupon.find('a', class_='coupon-link').attrs.get('data-code')
            expire_date = latest_coupon.find('span', class_='exp').contents[0]
            rules = ''
            has_rules = latest_coupon.find('div', class_='coupon-des-ellip').contents
            if has_rules:
                rules = has_rules[0]
            store_name = self._get_store_name()
            d[coupon] = dict(description=self._fix_description(description),
                             rules=rules,
                             store_name=store_name,
                             store_id='',
                             store_url='',
                             expire_date=self._fix_expire_date(expire_date))
        return d

    @classmethod
    def get_store_list(cls):
        d = {}
        ignore_url = 'https://www.cupomzeiros.com.br'
        header = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 '
                                '(KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        current_url = cls.base_url.format('/descontos/')
        r = requests.get(current_url, headers=header, timeout=60000)
        soup = BeautifulSoup(r.content, "lxml")
        letters = soup.find_all('ul', class_='clearfix')
        for letter in letters:
            for store_data in letter.find_all('li'):
                store_data = store_data.find('a')
                store_url = store_data.attrs.get('href').replace(ignore_url, '')
                store_name = store_data.contents[0].strip()
                d[store_name] = store_url
        return d
