# coding=utf-8

import requests
from bs4 import BeautifulSoup

from coupon.scrappers.basescrapper import all_stores, BaseScrapingClass, remove_accentuation
from coupon.scrappers.constants import URL_CUPONOMIA


class Cuponomia(BaseScrapingClass):
    base_url = 'https://www.cuponomia.com.br{}'

    def __init__(self, url):
        self.header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        r = requests.get(url, headers=self.header, timeout=60000)
        self.soup = BeautifulSoup(r.content, "lxml")
        self.blocked = self.soup.title.contents[0] == 'Desafio'
        self.valid_coupons = self.soup.find_all("ul", class_="coupon-list valid-coupons")

    def _is_valid_coupon(self, coupon):
        if 'NOMIA' in coupon.upper():
            return False
        return True

    @classmethod
    def get_url_for_store(cls, store_name):
        store_dict = all_stores.get(store_name)
        if not store_dict:
            return
        resource_url = store_dict.get(URL_CUPONOMIA)
        if not resource_url:
            return
        return cls.base_url.format(resource_url)

    # @classmethod
    # def get_url_for_category(cls, category_name):
    #     category_dict = feature_categories.get(category_name)
    #     if not category_dict:
    #         return
    #     return cls.base_url.format(category_dict.get(URL_CUPONOMIA))

    @classmethod
    def feature_store_list(cls):
        header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        stores_url = 'https://www.cuponomia.com.br/desconto'
        r = requests.get(stores_url, headers=header, timeout=60000)
        soup = BeautifulSoup(r.content, "html.parser")
        d = {}
        blocked = soup.title.contents[0] == 'Desafio'
        if blocked:
            return 'Serviço bloqueado'
        all_items = soup.findAll("ul", {"class": "featured-stores"})[0]
        for i in range(len(all_items) - 1):
            link = all_items.findAll(attrs={"class": "featured-item"})[i].contents[1].attrs.get('href')
            store_name = all_items.findAll(attrs={"class": "featured-item"})[i].contents[1].attrs.get('data-label')
            store_name = remove_accentuation(store_name.upper())
            d[store_name] = link
        return d

    @classmethod
    def all_store_list(cls):
        header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        stores_url = 'https://www.cuponomia.com.br/desconto'
        r = requests.get(stores_url, headers=header, timeout=60000)
        soup = BeautifulSoup(r.content, "html.parser")
        d = {}
        all_items = soup.findAll("li", {"class": "list-letter-item"})
        for i in all_items:
            link = i.contents[0].attrs.get('href')
            store_name = i.contents[0].contents[0].upper()
            d[store_name] = link
        return d

    @property
    def store_coupons(self):
        d = {}
        if self.blocked:
            return {'': {'description': 'bloqueado pelo cupomania'}, 'rules': ''}
        if len(self.valid_coupons) != 0:
            for tag in self.valid_coupons[0].contents:
                not_empty = False
                can_navigate = hasattr(tag, 'findAll')
                if can_navigate:
                    not_empty = tag.findAll(attrs={"class": "item-code-link"}) != []
                if not_empty:
                    tag_cupom = tag.findAll(attrs={"class": "item-code-link"})[0]
                    coupom = tag_cupom.contents[0]
                    store_name = tag.attrs.get('data-store-name')
                    store_id = tag.attrs.get('data-store-id')
                    tag_description = tag.findAll(attrs={"class": "item-title"})[0]
                    tag_expire = tag.find('li', class_='item-expiration i-header exp-soon')
                    expire_date = '...'
                    if tag_expire:
                        expire_date = tag_expire.contents[0].replace('Expira em ', '')
                    description = tag_description.contents[0]
                    pre_tag_rule = tag.findAll(attrs={"class": "item-desc"})
                    if len(pre_tag_rule) >= 1:
                        tag_rules = tag.findAll(attrs={"class": "item-desc"})[0]
                    else:
                        tag_rules = []
                    if len(tag_rules) == 3:
                        rules = tag_rules.contents[2]
                    elif len(tag_rules) == 1:
                        rules = tag_rules.contents[0]
                    else:
                        rules = '...'
                    if not store_id and not store_name:
                        continue
                    store_name = self._fix_store_name(store_name)
                    if self._is_valid_coupon(coupom):
                        d[coupom] = dict(description=description,
                                         rules=rules,
                                         store_name=store_name,
                                         expire_date=expire_date,
                                         store_url='',
                                         store_id=store_id)
        return d

    @classmethod
    def get_store_list(cls):
        d = {}
        header = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 '
                                '(KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        current_url = cls.base_url.format('/desconto')
        r = requests.get(current_url, headers=header, timeout=60000)
        soup = BeautifulSoup(r.content, "lxml")
        stores = soup.find_all('li', class_='list-letter-item')
        for store in stores:
            store_data = store.find('a')
            store_url = store_data.attrs.get('href')
            store_name = store_data.contents[0].strip()
            d[store_name] = store_url
        return d


class CuponomiaCategory(Cuponomia):
    @classmethod
    def feature_category_list(self):
        header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        categories_link = 'https://www.cuponomia.com.br/cupom'
        r = requests.get(categories_link, headers=header, timeout=60000)
        soup = BeautifulSoup(r.content, "html.parser")
        blocked = soup.title.contents[0] == 'Desafio'
        d = {}
        if blocked:
            return 'Serviço bloqueado'
        ul_class = 'category-list'
        t = soup.find("ul", class_=ul_class)
        for cat in t.contents:
            if hasattr(cat, 'contents'):
                category_desc = cat.contents[0].attrs.get('data-label').upper()
                category_link = cat.contents[0].attrs.get('href')
                category_desc = remove_accentuation(category_desc.upper())
                d[category_desc] = category_link
        return d

    @classmethod
    def all_category_list(cls):
        header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        categories_link = 'https://www.cuponomia.com.br/cupom'
        r = requests.get(categories_link, headers=header, timeout=60000)
        soup = BeautifulSoup(r.content, "html.parser")
        d = {}
        t = soup.findAll("li", class_='list-letter-item')
        for x in t:
            category_link = x.contents[0].attrs.get('href')
            category_name = x.contents[0].contents[0].upper()
            d[category_name] = category_link
        return d


class CuponomiaStore(Cuponomia):
    @property
    def store_id(self):
        class_section = 'item scroll-item staff-item item-code item-badge '
        class_section2 = 'item scroll-item item-code item-badge item-rewardable'
        atempt_1 = self.soup.find("li", class_=class_section)
        atempt_2 = self.soup.find("li", class_=class_section2)
        if atempt_1:
            return atempt_1.attrs.get('data-store-id')
        else:
            return atempt_2.attrs.get('data-store-id')

    @property
    def store_name(self):
        class_section = 'item scroll-item staff-item item-code item-badge '
        class_section2 = 'item scroll-item item-code item-badge item-rewardable'
        class_section3 = 'item scroll-item staff-item item-link item-badge item-rewardable'
        class_section4 = 'item scroll-item staff-item item-code item-badge item-rewardable'
        atempt_1 = self.soup.find("li", class_=class_section)
        atempt_2 = self.soup.find("li", class_=class_section2)
        atempt_3 = self.soup.find("li", class_=class_section3)
        atempt4 = self.soup.find("li", class_=class_section4)
        if atempt_1:
            return atempt_1.attrs.get('data-store-name')
        elif atempt_2:
            return atempt_2.attrs.get('data-store-name')
        elif atempt_3:
            return atempt_3.attrs.get('data-store-name')
        else:
            return atempt4.attrs.get('data-store-name')

    @property
    def store_logo(self):
        return self.soup.find("a", {"class": "store-logo"}).contents[1].get('src')
