import requests
from bs4 import BeautifulSoup

from coupon.scrappers.basescrapper import all_stores, BaseScrapingClass
from coupon.scrappers.constants import URL_GROUPON


class Groupon(BaseScrapingClass):
    base_url = 'https://www.groupon.com.br{}'

    def __init__(self, url):
        self.header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        r = requests.get(url, headers=self.header, timeout=60000)
        if r.status_code  == 404:
            return
        self.soup = BeautifulSoup(r.content, "html.parser")
        _valid_coupons = self.soup.find(id='coupons-list')
        self._caixa_cupons = _valid_coupons.findAll('div', class_='coupon-wrapper')

    def _get_store_name(self):
        store_name = self.soup.find('div', class_='row description-wrapper').find('h3').contents[0]
        store_name = self._fix_store_name(store_name)
        return store_name

    def _is_valid_coupon(self, coupon):
        return 'GROUPON' not in coupon.upper()

    @classmethod
    def get_url_for_store(cls, store_name):
        store_dict = all_stores.get(store_name)
        if not store_dict:
            return
        resource_url = store_dict.get(URL_GROUPON)
        if not resource_url:
            return
        return cls.base_url.format(resource_url)

    @classmethod
    def get_url_for_category(cls, category_name):
        return None

    @property
    def store_coupons(self):
        d = {}
        if not hasattr(self, '_caixa_cupons'):
            return {}
        for c in self._caixa_cupons:
            description = c.find('h4', class_='coupon-title').contents[0]
            rules = '...'
            has_restrictions = c.find('p', class_='columns twelve coupon-restrictions more toggle-text')
            if has_restrictions:
                rules = has_restrictions.find('span', class_='text').contents[0]
            expire_date = '...'
            has_expires = c.find('p', class_='expires')
            if has_expires:
                expire_date = has_expires.find('span').next_element.contents[0].replace(' BRT', '')
            has_coupon = c.find('div', class_='btn-cta reveal-code copy-button tooltip-trigger')
            if has_coupon:
                coupon = has_coupon.attrs.get('data-clipboard-text')
            else:
                continue
            store_name = self._get_store_name()
            if self._is_valid_coupon(coupon):
                d[coupon] = dict(description=self._fix_description(description),
                                 rules=rules,
                                 store_name=store_name,
                                 store_id='',
                                 store_url='',
                                 expire_date=expire_date)
        return d

    @classmethod
    def get_store_list(cls):
        d = {}
        letters = "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", \
                  "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "1", "2", "3", "4", \
                  "5", "6", "7", "8", "9"
        header = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 '
                                '(KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
        for letter in letters:
            current_url = cls.base_url.format('/starting-with/{}'.format(letter))
            r = requests.get(current_url, headers=header, timeout=60000)
            soup = BeautifulSoup(r.content, "lxml")
            stores = soup.find_all('li', class_='columns three end')
            for store in stores:
                store_data = store.find('a')
                store_url = store_data.attrs.get('href')
                store_name = store_data.contents[0].strip()
                d[store_name] = store_url
        return d
