import requests
from bs4 import BeautifulSoup

from coupon.scrappers.basescrapper import BaseScrapingClass, all_stores
from coupon.scrappers.constants import URL_PELANDO


class PelandoCoupons(BaseScrapingClass):
    base_url = 'https://www.pelando.com.br{}'

    def __init__(self, url):
        self.header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) '
                          'AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/50.0.2661.102 Safari/537.36'}
        session = self.login()
        r = session.get(url, headers=self.header, timeout=60000)
        self.soup = BeautifulSoup(r.content, "lxml")
        self._valid_coupons = self.soup.find_all('div',
                                                 class_='threadGrid')

    @staticmethod
    def login():
        session = requests.session()
        url_login = "https://www.pelando.com.br/login"

        payload = {
            'identity': 'marcelo.maia@live.com',
            'password': '12345678',
        }

        # Here, we're getting the login page and then grabbing hidden form
        # fields.  We're probably also getting several session cookies too.
        login = session.get(url_login)
        login_html = BeautifulSoup(login.content, 'html.parser')
        # hidden_inputs = login_html.xpath(r'//form//input[@type="hidden"]')
        hidden_inputs = login_html.find_all('input', type="hidden")
        csrf = {x["name"]: x["value"] for x in hidden_inputs}

        form = {**payload, **csrf}

        login_req = session.post(url_login, data=form)

        # Now to make sure you do not get the "Access denied", use the same
        # session variable for the other requests.
        return session

    def _get_store_name(self):
        store_name = self.soup.find('h1', class_='size--all-xxl text--b').contents[0].replace('Cupom de Desconto ', '')
        store_name = self._fix_store_name(store_name)
        return store_name

    def _is_valid_coupon(self, coupon):
        return 'PELANDO' not in coupon.upper()

    @classmethod
    def get_url_for_store(cls, store_name):
        store_dict = all_stores.get(store_name)
        if not store_dict:
            return
        resource_url = store_dict.get(URL_PELANDO)
        if not resource_url:
            return
        return cls.base_url.format(resource_url)

    @classmethod
    def get_url_for_category(cls, category_name):
        return None

    @property
    def store_coupons(self):
        d = {}
        for coupon_box in self._valid_coupons:
            has_description = coupon_box.find('span', class_=' space--r-1 size--all-s size--fromW3-m')
            if has_description:
                description = has_description.contents[0]
            else:
                continue
            has_coupon = coupon_box.find('span',
                                         class_='overflow--ellipsis width--all-12 hAlign--all-c text--color-brandPrimary text--b btn--mini')

            if has_coupon:
                coupon = has_coupon.contents[0]
            else:
                continue
            expire_date = ''

            rules = ''
            store_name = self._get_store_name()
            if self._is_valid_coupon(coupon):
                d[coupon] = dict(description=self._fix_description(description),
                                 rules=rules,
                                 store_name=store_name,
                                 store_id='',
                                 store_url='',
                                 expire_date=self._fix_expire_date(expire_date))
        return d

    @classmethod
    def get_store_list(cls):
        d = {}
        ignore_url = 'https://www.pelando.com.br'
        names_group = ['a-d', 'e-g', 'h-k', 'l-o', 'p-s', 't-w', 'x-z', '0-9']
        base_url = 'https://www.pelando.com.br/lojas/{}'
        header = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) '
                          'AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/50.0.2661.102 Safari/537.36'}
        for ng in names_group:
            url = base_url.format(ng)
            r = requests.get(url, headers=header, timeout=60000)
            soup = BeautifulSoup(r.content, 'lxml')
            stores = soup.find_all('li', class_='space--h-3 iGrid-item fGrid boxSec-space width--all-12'
                                                ' width--fromW2-6 width--fromW3-4 width--fromW4-3')
            for store in stores:
                url = store.find('a').attrs.get('href').replace(ignore_url, '')
                store_name = store.find('a').find('img').attrs.get('alt').strip()
                d[store_name] = url
        return d

    def _fix_description(self, description):
        return description.replace('\n', '').replace('\t', '')
