import unittest

import requests_cache

from coupon.scrappers.basescrapper import all_stores, remove_accentuation
from coupon.scrappers.cupomdedescontos1001 import CupomDesconto1001

requests_cache.install_cache('supercupomcache',
                             expire_after=90000)


class Test1001Store(unittest.TestCase):
    def test_store_name_coupons(self):
        """
        Verifica se o nome da loja obtido da pagina promocional bate com todos
        os store_name da respectiva loja
        """
        for store in all_stores.keys():
            url = CupomDesconto1001.get_url_for_store(store)
            if not url:
                continue
            cu = CupomDesconto1001(url)
            name = cu.store_name
            coupons = cu.store_coupons
            for coupon_data in coupons.values():
                yam_store_name = remove_accentuation(coupon_data['store_name']).upper().replace(' ', '')
                class_store_name = remove_accentuation(str(name)).upper().replace(' ', '')
                self.assertIn(yam_store_name, class_store_name)

    def test_store_coupon(self):
        """
        Verifica se todos os cupons obtidos são da mesma loja
         (valido somente para o metodo store_coupons)
        """
        for store in all_stores.keys():
            url = CupomDesconto1001.get_url_for_store(store)
            if not url:
                continue
            cu = CupomDesconto1001(url)
            store_names = [p['store_name'] for p in cu.store_coupons.values()]
            if len(store_names) >= 1:
                # check if all coupons are from same store
                self.assertEqual(store_names.count(store_names[0]), len(store_names))


if __name__ == '__main__':
    unittest.main()
